package eu.jonahbauer.raytracing.render.spectral;

import eu.jonahbauer.raytracing.render.spectral.colors.ColorSpace;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorXYZ;
import eu.jonahbauer.raytracing.render.spectral.spectrum.Spectrum;
import eu.jonahbauer.raytracing.render.texture.Color;
import org.jetbrains.annotations.NotNull;

public final class SampledSpectrum {
    private final double @NotNull[] values;


    public static void main(String[] args) {
        System.out.println();
    }

    public SampledSpectrum(@NotNull SampledWavelengths lambdas, @NotNull Spectrum spectrum) {
        var values = new double[lambdas.size()];
        for (int i = 0; i < values.length; i++) {
            values[i] = spectrum.get(lambdas.get(i));
        }
        this.values = values;
    }

    private SampledSpectrum(double @NotNull[] values) {
        this.values = values;
    }

    public static @NotNull SampledSpectrum multiply(@NotNull SampledSpectrum a, @NotNull SampledSpectrum b) {
        var out = new double[a.values.length];
        for (int i = 0; i < a.values.length; i++) {
            out[i] = a.values[i] * b.values[i];
        }
        return new SampledSpectrum(out);
    }

    public static @NotNull SampledSpectrum multiply(@NotNull SampledSpectrum a, double b) {
        var out = new double[a.values.length];
        for (int i = 0; i < a.values.length; i++) {
            out[i] = a.values[i] * b;
        }
        return new SampledSpectrum(out);
    }

    public static @NotNull SampledSpectrum add(@NotNull SampledSpectrum a, @NotNull SampledSpectrum b) {
        var out = new double[a.values.length];
        for (int i = 0; i < a.values.length; i++) {
            out[i] = a.values[i] + b.values[i];
        }
        return new SampledSpectrum(out);
    }

    public double get(int index) {
        return values[index];
    }

    public int size() {
        return values.length;
    }

    public double average() {
        double avg = 0;
        for (int i = 0; i < values.length; i++) {
            avg = Math.fma(1d / (i + 1), values[i] - avg, avg);
        }
        return avg;
    }

    public @NotNull ColorXYZ toXYZ(@NotNull SampledWavelengths lambdas) {
        return lambdas.toXYZ(this);
    }

    public @NotNull Color toRGB(@NotNull SampledWavelengths lambdas, @NotNull ColorSpace cs) {
        return cs.toRGB(toXYZ(lambdas));
    }
}
