package eu.jonahbauer.raytracing.render.spectral.colors;

import eu.jonahbauer.raytracing.math.Vec3;
import org.jetbrains.annotations.NotNull;

/**
 * A CIE XYZ color
 */
public record ColorXYZ(double x, double y, double z) {
    public static final double CIE_Y_INTEGRAL = 106.85689500000002;

    public ColorXYZ(@NotNull Chromaticity chromaticity) {
        this(chromaticity, 1);
    }

    public ColorXYZ(@NotNull Chromaticity chromaticity, double Y) {
        this(
                chromaticity.y() == 0 ? 0 : Y * chromaticity.x() / chromaticity.y(),
                chromaticity.y() == 0 ? 0 : Y,
                chromaticity.y() == 0 ? 0 : Y * (1 - chromaticity.x() - chromaticity.y()) / chromaticity.y()
        );
    }

    public ColorXYZ(@NotNull Vec3 vec) {
        this(vec.x(), vec.y(), vec.z());
    }

    public double average() {
        return (x + y + z) / 3;
    }

    public @NotNull Chromaticity xy() {
        var factor = 1 / (x + y + z);
        return new Chromaticity(factor * x, factor * y);
    }

    public @NotNull Vec3 toVec3() {
        return new Vec3(x, y, z);
    }

    public static @NotNull ColorXYZ multiply(@NotNull ColorXYZ a, @NotNull ColorXYZ b) {
        return new ColorXYZ(a.x * b.x, a.y * b.y, a.z * b.z);
    }

    public static @NotNull ColorXYZ multiply(@NotNull ColorXYZ a, double b) {
        return new ColorXYZ(a.x * b, a.y * b, a.z * b);
    }

    public static @NotNull ColorXYZ add(@NotNull ColorXYZ a, @NotNull ColorXYZ b) {
        return new ColorXYZ(a.x + b.x, a.y + b.y, a.z + b.z);
    }
}
