package eu.jonahbauer.raytracing.render.spectral.spectrum;

public final class BlackbodySpectrum implements Spectrum {
    /**
     * the speed of light in m/s
     */
    private static final double c = 299792458d;

    /**
     * the planck constant in m^2*kg/s
     */
    private static final double h = 6.62607015E-34;

    /**
     * the boltzmann constant in m^2*kg/s^2/K
     */
    private static final double k = 1.380649E-23;

    /**
     * wien's displacement constant in m*K
     */
    private static final double b = 2.897771995E-3;

    private final double T;
    private final double factor;

    public BlackbodySpectrum(double T) {
        if (T < 0) throw new IllegalArgumentException("T must be non-negative");
        this.T = T;
        this.factor = 1 / get(b / T);
    }

    @Override
    public double max() {
        return 1;
    }

    @Override
    public double get(double lambda) {
        lambda *= 1E-9;
        var l2 = lambda * lambda;
        var x = h * c / (lambda * k * T);
        return 2 * h * c * c / (l2 * l2 * lambda) / (Math.exp(x) - 1) * factor;
    }
}
