package eu.jonahbauer.raytracing.render.spectral.spectrum;

import java.util.Arrays;

public final class PiecewiseLinearSpectrum implements Spectrum {
    private final double[] lambdas;
    private final double[] values;
    private final double max;

    public PiecewiseLinearSpectrum(double[] lambdas, double[] values) {
        if (lambdas.length != values.length) {
            throw new IllegalArgumentException("lambdas and values must have the same length");
        }

        this.lambdas = Arrays.copyOf(lambdas, lambdas.length);
        this.values = Arrays.copyOf(values, values.length);

        var max = 0d;
        for (int i = 1; i < this.lambdas.length; i++) {
            if (this.lambdas[i] <= this.lambdas[i - 1]) {
                throw new IllegalArgumentException("lambdas must be in increasing order");
            }
            if (this.values[i] < 0) {
                throw new IllegalArgumentException("values must be non-negative");
            } else if (this.values[i] > max) {
                max = this.values[i];
            }
        }
        this.max = max;
    }

    @Override
    public double max() {
        return max;
    }

    @Override
    public double get(double lambda) {
        if (lambdas.length == 0 || lambda < lambdas[0] || lambda > lambdas[lambdas.length - 1]) return 0;
        if (lambda == lambdas[lambdas.length - 1]) return values[values.length - 1];

        var i = Arrays.binarySearch(lambdas, lambda);
        if (i < 0) i = -i - 1;

        var t = (lambda - lambdas[i]) / (lambdas[i + 1] - lambdas[i]);
        return (1 - t) * values[i] + t * values[i + 1];
    }
}
