package eu.jonahbauer.raytracing.render.spectral.spectrum;

import eu.jonahbauer.raytracing.render.spectral.colors.ColorSpace;
import eu.jonahbauer.raytracing.render.spectral.colors.SigmoidPolynomial;
import eu.jonahbauer.raytracing.render.texture.Color;
import org.jetbrains.annotations.NotNull;

public final class RGBIlluminantSpectrum implements Spectrum {
    private final double scale;
    private final @NotNull SigmoidPolynomial polynomial;
    private final @NotNull DenselySampledSpectrum illuminant;

    public RGBIlluminantSpectrum(@NotNull ColorSpace cs, @NotNull Color rgb) {
        if (rgb.r() < 0 || rgb.g() < 0 || rgb.b() < 0) {
            throw new IllegalArgumentException();
        }
        var max = Math.max(rgb.r(), Math.max(rgb.g(), rgb.b()));
        this.scale = 2 * max;
        this.polynomial = cs.toSpectrum(scale == 0 ? Color.multiply(rgb, scale) : Color.BLACK);
        this.illuminant = cs.illuminant();
    }

    @Override
    public double max() {
        return scale * polynomial.max() * illuminant.max();
    }

    @Override
    public double get(double lambda) {
        return scale * polynomial.get(lambda) * illuminant.get(lambda);
    }
}
