package eu.jonahbauer.raytracing.render.spectral.spectrum;

import eu.jonahbauer.raytracing.render.spectral.colors.ColorSpace;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorXYZ;
import eu.jonahbauer.raytracing.render.spectral.SampledSpectrum;
import eu.jonahbauer.raytracing.render.spectral.SampledWavelengths;
import eu.jonahbauer.raytracing.render.texture.Color;
import org.jetbrains.annotations.NotNull;

public interface Spectrum {
    int LAMBDA_MIN = 360;
    int LAMBDA_MAX = 830;
    int SAMPLES = 4;

    /**
     * {@return the maximum value of <code>this</code> spectrum over the range of wavelengths}
     */
    double max();

    /**
     * {@return the value of <code>this</code> spectrum at a given wavelength}
     * @param lambda the wavelength in nanometers
     */
    double get(double lambda);

    default @NotNull SampledSpectrum sample(@NotNull SampledWavelengths lambdas) {
        return new SampledSpectrum(lambdas, this);
    }

    default @NotNull ColorXYZ toXYZ() {
        return new ColorXYZ(
                innerProduct(Spectra.X, this) / ColorXYZ.CIE_Y_INTEGRAL,
                innerProduct(Spectra.Y, this) / ColorXYZ.CIE_Y_INTEGRAL,
                innerProduct(Spectra.Z, this) / ColorXYZ.CIE_Y_INTEGRAL
        );
    }

    default @NotNull Color toRGB(@NotNull ColorSpace cs) {
        return cs.toRGB(toXYZ());
    }

    private static double innerProduct(@NotNull Spectrum f, @NotNull Spectrum g) {
        var integral = 0.0;
        for (var lambda = LAMBDA_MIN; lambda <= LAMBDA_MAX; lambda++) {
            integral += f.get(lambda) * g.get(lambda);
        }
        return integral;
    }
}
